#!/usr/bin/perl -w

# Read test data sets for Cell-line use case and Sanofi RC2 micro RNA
# create subset of MirBase for only these entries for rapid loading.

# TO DO
# =====
#
# process aliases
# handle species e.g. rno-miR-207 mmu-miR-207 hsa-mir-207 (if present)

open(TESTDATA, "mirna_data_list.txt") || die "mirna_data_list not found";

while(<TESTDATA>) {
    s/[\#].*//;
    if(/^(\S.*\S)$/){
	$mfilename = $1;
	open(MFILE, "$mfilename") || die "Failed to read micro RNA file '$mfilename'";
	$mline = <MFILE>;		# skip headers
	while($mline = <MFILE>){
	    @col = split(/\t/,$mline);
	    $id = lc($col[1]);
	    $id =~ s/\"//g;
	    $id =~ s/[*]$//;
	    $id =~ s/[*]$//;
	    $id =~ s/[\#]$//;
	    if($id eq "") {next} # skip entries with no ID
	    if($id =~ /(candidate\S+) [\\(]([^\\)]+)[\\)]/) {
#		++$wantid{$1};
		$id = $2;
	    }
	    if($id =~ /^mi/){$id = "hsa-$id"}
	    if($id =~ /^let-/){$id = "hsa-$id"}
	    ++$wantid{$id};
	    $source{$id} = $mfilename;
	}
	close MFILE;
    }
}
close TESTDATA;

$wantentries = keys(%wantid);

print "Reading /data/scratch/git-master/dictionaries/Data/MiRBase/aliases.txt\n";
open (DIC, "/data/scratch/git-master/dictionaries/Data/MiRBase/aliases.txt") || die "Cannot open aliases";
$line = 0;

while($dline = <DIC>){
    ++$line;
    chomp($dline);
    $dline =~ s/;$//;
    @col = split(/\t/,$dline);
    $accnum = $col[0];
    $idlist = $col[1];
    if(!defined($wantid{$accnum})){
	if($idlist ne ""){
	    @id=split(/[; ]/,$idlist);
	    foreach $id (@id) {
		$id =~ s/[*]//;
		$id =~ s/[\#]//;
		if(defined($wantid{$id})){
		    $wantalias{$accnum} = $id;
#		    print STDERR "accnum $accnum id $id ($idlist)\n";
		}
	    }
	}
    }
}
close DIC;
$wantaliases = keys(%wantalias);

print "Reading /data/scratch/git-master/dictionaries/Data/MiRBase/miRNA.dead\n";
open (DEAD, "/data/scratch/git-master/dictionaries/Data/MiRBase/miRNA.dead") || die "Cannot open MiRNA.dead";

while(<DEAD>) {
    if(/^AC +(\S+)/) {$dead{$1}++}
    if(/^ID +(\S+)/) {$dead{$1}++}
    if(/^PI +(\S+)/) {
	$pi = $1;
	foreach $p (split(/ /,$pi)) {
	    $dead{$p}++;
	}
    }
}
close DEAD;

$deaduns = keys(%dead);

print "Reading /data/scratch/git-master/dictionaries/Data/MiRBase/miRNA.dat\n";
open (DIC, "/data/scratch/git-master/dictionaries/Data/MiRBase/miRNA.dat") || die "Cannot open miRNA.dat";
$line = 0;
$saveentry=0;

while($dline = <DIC>){
    ++$line;

    if($dline =~ /^ID   (\S+)/) {
	$text = $dline;
	$id = lc($1);
	$accnum = "";
	$wanted = 0;
	if(defined($wantid{$id})){
	    $wanted = 1;
	    $gotid{$id}=1;
	}
	if($id =~ /^(\S+)-1$/) {
	    if(defined($wantid{$1})){
		$wanted=1;
		$gotid{$id}=1;
	    }
	}
#	print STDERR "id $id\n";
    }
    elsif($dline =~ /^\/\//){
	if($wanted) {
	    print $text;
	    print $dline;
	    ++$saveentry;
#	    if(defined($wantid{$id})){print STDERR "Wanted id $id $wantid{$id} "}
#	    if(defined($wantid{$accnum})){print STDERR "Wanted accnum $accnum $wantid{$accnum} "}
	    #	    print STDERR " found\n";
	}
    }
    else {
	if($dline =~ /^AC   ([^;]+);/){
	    $accnum = $1;
	    if(defined($wantalias{$accnum})) {
		$wanted = 1;
		$gotid{$wantalias{$accnum}}=1;
	    }
#	    print STDERR "accnum $accnum\n";
	}
	if($dline =~ /^DR +(.*)[.;]$/){
	    $dr = $1;
	    $dr =~ s/ //g;
	    @dr = split(/[;]/,$dr);
	    print STDERR "id $id test DR '$dr'\n";
	    foreach $d (@dr) {
		if(defined($wantalias{$d})) {
		    $wanted = 1;
		    $gotid{$wantalias{$d}}=1;
		    print STDERR "wanted dbreference '$d'\n";
		}
		if(defined($wantid{$d})) {
		    $wanted = 1;
		    $gotid{$d}=1;
		    print STDERR "wanted dbreference '$d'\n";
		}
	    }
#	    print STDERR "unwanted accession '$acc'\n";
	}
	if($dline =~ /FT +\/product=\"([^\"]+)\"/){
	    $acc = $1;
	    if(defined($wantalias{$acc})) {
		$wanted = 1;
		$gotid{$wantalias{$acc}}=1;
		print STDERR "wanted accession '$acc'\n";
	    }
#	    print STDERR "unwanted accession '$acc'\n";
	}
	if($dline =~ /FT +\/product=\"([^\"]+)\"/){
	    $product = lc($1);
	    if(defined($wantid{$product})) {
		$wanted=1;
		$gotid{$product}=1;
#		print STDERR "wanted product '$product'\n";
	    }
#	    print STDERR "unwanted product '$product'\n";
	}
	$text .= $dline;
    }
}
close DIC;

print STDERR "Wanted $wantentries aliases $wantaliases dead $deaduns found $saveentry\n";

open (MISS,">mirna-missing.txt") || die "failed to open output file missing.txt";
$missing = 0;
foreach $id (keys(%wantid)){
    if(!defined($gotid{$id})) {
	if(defined($dead{$id})){
#	    print STDERR "Dead ID $id $source{$id}\n";
	    print STDERR "Dead ID $id\n";
	}
	else {
	    print STDERR "Missing ID $id $source{$id}\n";#
	    print MISS "$id\n";
	    $missing++;
	}
    }
}
close MISS;
foreach $d (keys(%dead)) {
    print STDERR "dead '$d'\n";
}

print STDERR "Total $missing missing\n";
